"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/* tslint:disable:no-console */
const apputils_1 = require("@jupyterlab/apputils");
const mainmenu_1 = require("@jupyterlab/mainmenu");
const cells_1 = require("@jupyterlab/cells");
const notebook_1 = require("@jupyterlab/notebook");
const coreutils_1 = require("@jupyterlab/coreutils");
const services_1 = require("@jupyterlab/services");
const fileeditor_1 = require("@jupyterlab/fileeditor");
const disposable_1 = require("@phosphor/disposable");
require("../style/index.css");
const PLUGIN_NAME = "jupyterlab_code_formatter";
const FORMAT_COMMAND = "jupyterlab_code_foramtter:format";
const FORMAT_ALL_COMMAND = "jupyterlab_code_foramtter:format_all";
function request(path, method, body, settings) {
    const fullUrl = coreutils_1.URLExt.join(settings.baseUrl, PLUGIN_NAME, path);
    return services_1.ServerConnection.makeRequest(fullUrl, { body, method }, settings).then((response) => {
        if (response.status !== 200) {
            return response.text().then((data) => {
                throw new services_1.ServerConnection.ResponseError(response, data);
            });
        }
        return response.text();
    });
}
class JupyterLabCodeFormatter {
    constructor(app, tracker, palette, settingRegistry, menu, editorTracker) {
        this.working = false;
        this.pythonCommands = ["black", "yapf", "autopep8", "isort"].map((name) => `${PLUGIN_NAME}:${name}`);
        this.rCommands = ["formatR", "styler"].map((name) => `${PLUGIN_NAME}:${name}`);
        this.app = app;
        this.tracker = tracker;
        this.editorTracker = editorTracker;
        this.palette = palette;
        this.settingRegistry = settingRegistry;
        this.menu = menu;
        this.setupSettings();
        request("formatters", "GET", null, services_1.ServerConnection.defaultSettings).then((data) => {
            const formatters = JSON.parse(data).formatters;
            const menuGroup = [];
            Object.keys(formatters).forEach((formatter) => {
                if (formatters[formatter].enabled) {
                    const command = `${PLUGIN_NAME}:${formatter}`;
                    this.setupButton(formatter, formatters[formatter].label, command);
                    menuGroup.push({ command });
                }
            });
            this.menu.editMenu.addGroup(menuGroup);
        });
    }
    createNew(nb, context) {
        const btn = new apputils_1.CommandToolbarButton({
            commands: this.app.commands,
            id: FORMAT_ALL_COMMAND,
        });
        nb.toolbar.insertAfter("cellType", this.app.commands.label(FORMAT_ALL_COMMAND), btn);
        return new disposable_1.DisposableDelegate(() => {
            btn.dispose();
        });
    }
    getCodeCells(selectedOnly = true) {
        if (!this.tracker.currentWidget) {
            return [];
        }
        const cells = [];
        const notebook = this.tracker.currentWidget.content;
        notebook.widgets.forEach((cell) => {
            if (cell.model.type === "code") {
                if (!selectedOnly || notebook.isSelectedOrActive(cell)) {
                    cells.push(cell);
                }
            }
        });
        return cells;
    }
    formatSelectedCodeCells() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._formatCells(true);
        });
    }
    formatAllCodeCells() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._formatCells(false);
        });
    }
    getDefaultFormatter() {
        const notebookType = this.getNotebookType();
        if (notebookType) {
            return this.config.preferences.default_formatter[notebookType];
        }
        return null;
    }
    _formatCells(selectedOnly) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.working) {
                return;
            }
            try {
                this.working = true;
                const selectedCells = this.getCodeCells(selectedOnly);
                if (selectedCells.length === 0) {
                    this.working = false;
                    return;
                }
                const currentTexts = selectedCells.map((cell) => cell.model.value.text);
                const formatter = this.getDefaultFormatter();
                const formattedTexts = yield this.formatCode(currentTexts, formatter);
                for (let i = 0; i < selectedCells.length; ++i) {
                    const cell = selectedCells[i];
                    const currentText = currentTexts[i];
                    const formattedText = formattedTexts.code[i];
                    if (cell.model.value.text === currentText) {
                        if (formattedText.code) {
                            cell.model.value.text = formattedText.code;
                        }
                        else {
                            console.error("Could not format cell: %s due to:\n%o", currentText, formattedText.error);
                        }
                    }
                    else {
                        console.error("Value changed since we formatted - skipping: %s", cell.model.value.text);
                    }
                }
            }
            catch (err) {
                console.error("Something went wrong :(\n%o", err);
            }
            this.working = false;
        });
    }
    getNotebookType() {
        if (this.tracker.currentWidget) {
            const metadata = this.tracker.currentWidget.content.model.metadata.toJSON();
            if (metadata && metadata.kernelspec) {
                // @ts-ignore
                return metadata.kernelspec.language;
            }
        }
        return null;
    }
    setupSettings() {
        const self = this;
        Promise.all([this.settingRegistry.load(`@ryantam626/${PLUGIN_NAME}:settings`)]).then(([settings]) => {
            function onSettingsUpdated(jsettings) {
                self.config = jsettings.composite;
            }
            settings.changed.connect(onSettingsUpdated);
            onSettingsUpdated(settings);
        }).catch((reason) => console.error(reason.message));
    }
    maybeFormatCodecell(formatterName) {
        // TODO: Check current kernel is of appropriate kernel
        const editorWidget = this.editorTracker.currentWidget;
        if (this.working) {
            return;
        }
        if (editorWidget && editorWidget.content !== null && editorWidget.content.isVisible) {
            this.working = true;
            const editor = editorWidget.content.editor;
            const code = editor.model.value.text;
            this.formatCode([code], formatterName).then((data) => {
                if (data.code[0].error) {
                    throw data.code[0].error;
                }
                this.editorTracker.currentWidget.content.editor.model.value.text = data.code[0].code;
                this.working = false;
            }).catch((err) => {
                this.working = false;
                console.error("Something went wrong :(:\n%o", err);
            });
        }
        else if (this.tracker.activeCell instanceof cells_1.CodeCell) {
            this.working = true;
            const code = this.tracker.activeCell.model.value.text;
            this.formatCode([code], formatterName).then((data) => {
                if (data.code[0].error) {
                    throw data.code[0].error;
                }
                this.tracker.activeCell.model.value.text = data.code[0].code;
                this.working = false;
            }).catch((err) => {
                this.working = false;
                console.error("Something went wrong :(:\n%o", err);
            });
        }
    }
    formatCode(code, formatter) {
        return request("format", "POST", JSON.stringify({
            code,
            formatter,
            options: this.config[formatter],
        }), services_1.ServerConnection.defaultSettings).then((resp) => JSON.parse(resp));
    }
    setupButton(name, label, command) {
        this.app.commands.addCommand(command, {
            execute: () => {
                this.maybeFormatCodecell(name);
            },
            isVisible: () => {
                const widget = this.app.shell.currentWidget;
                // TODO: handle other languages other than Python
                const editorWidget = this.editorTracker.currentWidget;
                const notebookWidget = this.tracker.currentWidget;
                return widget && ((this.pythonCommands.some((cmd) => cmd === command) && editorWidget && widget === editorWidget &&
                    coreutils_1.PathExt.extname(editorWidget.context.path).toLowerCase() === ".py") ||
                    (this.rCommands.some((cmd) => cmd === command) && editorWidget && widget === editorWidget &&
                        coreutils_1.PathExt.extname(editorWidget.context.path).toLowerCase() === ".r") ||
                    (notebookWidget && widget === notebookWidget));
            },
            label,
        });
        this.palette.addItem({ command, category: "JupyterLab Code Formatter" });
    }
}
/**
 * Initialization data for the jupyterlab_code_formatter extension.
 */
const extension = {
    activate: (app, palette, tracker, settingRegistry, menu, editorTracker) => {
        const jlcf = new JupyterLabCodeFormatter(app, tracker, palette, settingRegistry, menu, editorTracker);
        app.docRegistry.addWidgetExtension("Notebook", jlcf);
        app.commands.addCommand(FORMAT_COMMAND, {
            execute: () => __awaiter(this, void 0, void 0, function* () {
                yield jlcf.formatSelectedCodeCells();
            }),
            isVisible: () => jlcf.getDefaultFormatter() && jlcf.getCodeCells().length > 0,
            label: "Format cell",
        });
        app.commands.addCommand(FORMAT_ALL_COMMAND, {
            execute: () => __awaiter(this, void 0, void 0, function* () {
                yield jlcf.formatAllCodeCells();
            }),
            isVisible: () => jlcf.getDefaultFormatter(),
            label: "Format notebook",
        });
        app.contextMenu.addItem({ command: FORMAT_COMMAND, selector: ".jp-Notebook" });
    },
    autoStart: true,
    id: PLUGIN_NAME,
    requires: [apputils_1.ICommandPalette, notebook_1.INotebookTracker, coreutils_1.ISettingRegistry, mainmenu_1.IMainMenu, fileeditor_1.IEditorTracker],
};
exports.default = extension;
